@@ -1,8 +1,8 @@
 /***
-https://raw.github.com/jdlrobson/ts.js/master/src/ts.js
+https://raw.github.com/TiddlySpace/ts.js/master/src/ts.js
 ***/
 //{{{
-// version 0.4.0
+// version 0.5.3
 (function() {
 var getCSRFToken = function(window) {
 	// XXX: should not use RegEx - cf.
@@ -73,18 +73,19 @@ var ts = {
 			$(".inputArea", form).show();
 		}
 	},
-	resolveCurrentSpaceName: function(options) {
+	resolveCurrentSpaceName: function(options, host) {
 		if(options && typeof(options.space) != "undefined") {
 			ts.currentSpace = options.space;
 		} else if(window.location.protocol !== "file:") {
-			ts.currentSpace = window.location.hostname.split(".")[0];
+			if(host.split(".").length < window.location.hostname.split(".").length) {
+				ts.currentSpace = window.location.hostname.split(".")[0];
+			}
 		}
 	},
 	isValidSpaceName: function(name) {
 		return name.match(/^[a-z][0-9a-z\-]*[0-9a-z]$/) ? true : false;
 	},
 	init: function(callback, options) {
-		ts.resolveCurrentSpaceName(options);
 		ts.loadHash();
 		var register = $("form.ts-registration").addClass("tsInitializing")[0];
 		var login = $("form.ts-login").addClass("tsInitializing")[0];
@@ -94,12 +95,16 @@ var ts = {
 		$("input[type=submit]", [login, logout, register, openid]).attr("disabled", true);
 		$.ajax({ url: "/status", 
 			success: function(status) {
+				ts.resolveCurrentSpaceName(options, status.server_host.host);
+				if(!ts.currentSpace) {
+					$(document.body).addClass("ts-unknown-space");
+				}
 				$(register).removeClass("tsInitializing");
 				$(login).removeClass("tsInitializing");
 				$(logout).removeClass("tsInitializing");
 				$("input[type=submit]", [login, logout, register, openid]).attr("disabled", false);
 				ts.loadStatus(status);
-				if(status.identity) {
+				if(status.identity || ts.parameters.openid) {
 					ts.register_openid(status.identity);
 				} else if(status.username && ts.parameters.openid) {
 					// open id login occurred so redirect to homespace
@@ -196,9 +201,18 @@ var ts = {
 			}
 		}
 	},
+	_register_openid_for_user: function(username, openid) {
+		var user = new tiddlyweb.User(username, null, ts.getHost(ts.currentSpace));
+		user.identities().add(openid, function() {
+			window.location.href = window.location.pathname;
+		}, function() {
+			throw "failed to add identity to current user.";
+		});
+	},
 	register_openid: function(openid) {
 		var space = ts.parameters.space;
-		if(!space) {
+		var username = ts.parameters.user;
+		if(!space && !username) {
 			var answer = confirm("Would you like to create a space with your openid: " + openid  + "?");
 			if(answer) {
 				space = prompt("What would you like to be your TiddlySpace username?");
@@ -219,17 +233,7 @@ var ts = {
 					// login as that newly created user
 					ts.login(space, password, {
 						success: function() {
-							// map user back to the openid
-							var tiddler = new tiddlyweb.Tiddler(openid);
-							tiddler.bag = new tiddlyweb.Bag("MAPUSER", "/");
-							var callback = function(data, status, xhr) {
-								// do redirect
-								window.location.href = ts.parameters.redirect || ts.getHost(space);
-							};
-							var errback = function(r) {
-								throw "failed at step 3/3";
-							};
-							tiddler.put(callback, errback);
+							ts._register_openid_for_user(space, ts.parameters.openid);
 						},
 						errback: function() {
 							throw "failed at step 2/3";
@@ -237,6 +241,8 @@ var ts = {
 					});
 				}
 			});
+		} else if(username) {
+			ts._register_openid_for_user(username, ts.parameters.openid);
 		}
 	},
 	register: function(username, password, form, options) {
@@ -287,7 +293,12 @@ var ts = {
 	},
 	lists: {
 		identities: function() {
-			var list = $("ul.ts-identities").addClass("ts-loading")[0];
+			var list = $("ul.ts-identities")[0];
+			if(!list) {
+				return;
+			} else {
+				$(list).addClass("ts-loading");
+			}
 			var user = new tiddlyweb.User(ts.user.name, null, "/");
 			user.identities().get(
 				function(identities) {
@@ -492,22 +503,28 @@ var ts = {
 			$(form).attr("method", "post").
 				attr("action", "/challenge/tiddlywebplugins.tiddlyspace.openid").
 				submit(function(ev) {
-				var user = $("input[name=openid]", form).val();
+				var identity = $("input[name=openid]", form).val();
 				var space = $("input[name=space]", form).val();
-				if(!user) {
+				var user = options && options.user ? options.user.name : null;
+				if(!identity) {
 					ev.preventDefault();
 					return ts.messages.display(form, "Please provide an openid!");
 				}
-				var querystring = "?openid=" + user;
+				var querystring = "?openid=" + identity;
 				if(space) {
 					querystring += "&space=" + space;
 				}
+				if(user) {
+					querystring += "&user=" + user;
+				}
 				var redirect = $("[name=redirect]", form).val();
 				if(redirect) {
 					querystring += "&redirect=" + redirect;
 				}
+				// IMPORTANT: #auth:OpenID=<openid> is read by the openid tiddlyweb plugin
+				// when present it keeps you logged in as your cookie username
 				$('<input name="tiddlyweb_redirect" type="hidden" />').
-					val(window.location.pathname + querystring).appendTo(form);
+					val(window.location.pathname + querystring + "#auth:OpenID=" + identity).appendTo(form);
 			});
 		},
 		logout: function(container) {
@@ -545,22 +562,22 @@ var ts = {
 		}
 	},
 	loginStatus: function(login, register, logout) {
-		var openid = $("form.ts-openid");
 		var status = ts.status;
 		var user = ts.user;
+		$("form.ts-openid").each(function(i, el) {
+			ts.forms.openid(el, { user: user });
+		});
 		if(!user.anon) {
 			$(document.body).addClass("ts-loggedin");
 			$([register, login]).remove();
-			openid.each(function(i, el) {
-				$(el).remove();
-			});
 			if(!logout) {
 				return;
 			}
 			$(logout).empty();
 			var container = logout;
 			var uri = ts.getHost(user.name);
-			var link = $("<a />").attr("href", uri).text(user.name)[0];
+			var link = $("<a />").attr({"href": uri,
+				"target": "_parent"}).text(user.name)[0];
 			$("<span />").text("Welcome back ").appendTo(container);
 			$(container).append(link);
 			$("<span />").text("!").appendTo(container);
@@ -573,11 +590,6 @@ var ts = {
 				ts.forms.login(login);
 			}
 
-			if(openid) {
-				openid.each(function(i, el) {
-					ts.forms.openid(el);
-				});
-			}
 			$(logout).remove();
 		}
 	}
